@@ -62,9 +62,8 @@
  * Some systems will define ushort in sys/types.h, others will not.
  * Unfortunately there is no #define symbol to key on;
  * no way to conditionally compile the following statement. */
-#ifdef DOSLIKE
+#if defined(DOSLIKE) || defined(__ANDROID__)
 typedef unsigned short ushort;
-typedef unsigned long ulong;
 #endif
 /* sys/types.h defines unsigned char as unchar.  I prefer uchar.
  * It is consistent with ushort uint and ulong, and doesn't remind
@@ -82,9 +81,6 @@ typedef uchar bool;
 // between edbrowse and the js engine.
 typedef void *jsobjtype;
 
-extern jsobjtype jcx; // the javascript context
-extern jsobjtype winobj;	// window object
-extern jsobjtype docobj;	// document object
 extern const char *jsSourceFile; // sourcefile providing the javascript
 extern int jsLineno; // line number
 
@@ -108,6 +104,7 @@ enum ej_proptype {
 	EJ_PROP_ARRAY,
 	EJ_PROP_FUNCTION,
 	EJ_PROP_INSTANCE,
+	EJ_PROP_NULL,
 };
 
 /* ctype macros, when you're passing a byte,
@@ -145,7 +142,7 @@ typedef uchar *pst;		/* perl string */
 #define MAXTTYLINE 256 // max length of an entered line
 #define MAXHOSTLEN 400
 #define MAXPROTLEN 12
-#define MAXUSERPASS 40 // user name or password
+#define MAXUSERPASS 80 // user name or password
 #define MAXACCOUNT 100 // number of email accounts
 #define MAXAGENT 50 // number of user agents
 #define MAXMIME 40 // number of mime types
@@ -290,7 +287,7 @@ extern char *sslCerts;		/* ssl certificates to validate the secure server */
 extern int verifyCertificates;	/* is a certificate required for the ssl connection? */
 extern int displayLength;	// when printing a line
 extern int formatLineLength;	// when formatting html
-bool formatOverflow;
+extern bool formatOverflow;
 extern int webTimeout, mailTimeout;
 extern uchar browseLocal;
 extern bool sqlPresent;		/* Was edbrowse compiled with SQL built in? */
@@ -389,6 +386,7 @@ struct ebFrame {
 	struct ebFrame *next;
 	struct ebWindow *owner;
 	struct htmlTag *frametag;
+	int gsn; // global sequence number
 	char *fileName;		/* name of file or url */
 	char *firstURL;		// before http redirection
 	char *hbase; /* base for href references */
@@ -404,18 +402,17 @@ struct ebFrame {
 /* The javascript context and window corresponding to this url or frame.
  * If this is null then javascript is not operational for this frame.
  * We could still be browsing however, without javascript. */
-	jsobjtype jcx;
+	jsobjtype cx;
 	jsobjtype winobj;
 	jsobjtype docobj;	/* window.document */
 	const struct MIMETYPE *mt;
 	void *cssmaster;
 	short jtmin;
 };
-extern struct ebFrame *cf;	/* current frame */
 
-#define set_js_globals_f(f) (jcx = f->jcx, winobj = f->winobj, docobj = f->docobj)
-#define set_js_globals() set_js_globals_f(cf)
-#define get_js_globals() (cf->jcx = jcx, cf->winobj = winobj, cf->docobj = docobj)
+typedef struct ebFrame Frame;
+extern Frame *cf;	/* current frame */
+extern int gfsn; // global frame sequence number
 
 /* single linked list for internal jump history */
 struct histLabel {
@@ -455,7 +452,7 @@ struct ebWindow {
  * and used thereafter for hyperlinks, fill-out forms, etc. */
 	struct htmlTag **tags;
 	int numTags, allocTags, deadTags;
-	struct htmlTag *scriptlist, *inputlist, *optlist;
+	struct htmlTag *scriptlist, *inputlist, *optlist, *linklist;
 	bool mustrender:1;
 	bool sank:1; /* jSyncup has been run */
 	bool lhs_yes:1;
@@ -488,7 +485,7 @@ extern struct ebWindow *cw;	/* current window */
 #define tagList (cw->tags)
 
 /* js is running in the current session */
-#define isJSAlive (cf->jcx != NULL && allowJS)
+#define isJSAlive (cf->cx && allowJS)
 
 /*********************************************************************
 Temporary cap on the number of lines, so the integer index into cw->map
@@ -541,6 +538,7 @@ struct htmlTag {
 	jsobjtype jv;		/* corresponding javascript variable */
 	jsobjtype style; // style object
 	int seqno;
+	int gsn; // global sequence number, for rooting
 	char *js_file;
 	int js_ln;			/* line number of javascript */
 	int lic;		/* list item count, highly overloaded */
@@ -573,7 +571,8 @@ struct htmlTag {
 	bool checked:1;
 	bool rchecked:1;	/* for reset */
 	bool post:1;		/* post, rather than get */
-	bool javapost:1;	/* post by calling javascript */
+	bool javapost:1;	// post by calling javascript
+	bool jslink:1;	// linked to a js object
 	bool mime:1;		/* encode as mime, rather than url encode */
 	bool bymail:1;		/* send by mail, rather than http */
 	bool submitted:1;
@@ -588,7 +587,7 @@ struct htmlTag {
 	bool masked:1;
 	bool iscolor:1;
 	char subsup;		/* span turned into sup or sub */
-	uchar itype;		/* input type = */
+	uchar itype;		// input type =
 	uchar itype_minor;
 #define DIS_INVISIBLE 1
 #define DIS_HOVER 2
@@ -597,13 +596,16 @@ struct htmlTag {
 #define DIS_HOVERCOLOR 5
 	uchar disval; // displayable value for the node
 	int ninp;		/* number of nonhidden inputs */
-	char *name, *id, *class, *nodeName, *value, *href;
+// class is reserved word in c++, so use jclass for javascript class
+	char *name, *id, *jclass, *nodeName, *value, *href;
 	const char *rvalue; /* for reset */
 	char *innerHTML; /* the html string under this tag */
 	int inner;		/* for inner html */
 	int highspec; // specificity of a selector that matches this node
 };
 
+typedef struct htmlTag Tag;
+
 /* htmlTag.action */
 enum {
 	TAGACT_HTML, TAGACT_A, TAGACT_INPUT, TAGACT_TITLE, TAGACT_TA,
