@@ -1,5 +1,15 @@
 /* Prototypes for edbrowse */
 
+/*********************************************************************
+Some time ago we used the Mozilla js engine, which is in c++.
+If we ever go back to that, or use v8 or any other c++ engine,
+we need to indicate that all the other functions are straight C.
+*********************************************************************/
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* sourcefile=auth.c */
 bool getUserPass(const char *url, char *creds, bool find_proxy) ;
 bool getUserPassRealm(const char *url, char *creds, const char *realm);
@@ -67,41 +77,41 @@ int goSelect(int *startLine, char **rbuf) ;
 void javaSetsTagVar(jsobjtype v, const char *newtext);
 void dwStart(void);
 void createJavaContext(void) ;
-void freeJavaContext(struct ebFrame *f) ;
-char *jsRunScriptResult(jsobjtype obj, const char *str, const char *filename, int lineno) ;
-void jsRunScript(jsobjtype obj, const char *str, const char *filename, int lineno) ;
-void jsRunData(jsobjtype obj, const char *filename, int lineno);
-enum ej_proptype typeof_property(jsobjtype obj, const char *name) ;
-bool has_property(jsobjtype obj, const char *name) ;
-#define handlerPresent(obj, name) (typeof_property(obj, name) == EJ_PROP_FUNCTION)
-void delete_property(jsobjtype obj, const char *name) ;
-char *get_property_string(jsobjtype obj, const char *name) ;
-int get_property_number(jsobjtype obj, const char *name) ;
-double get_property_float(jsobjtype obj, const char *name) ;
-bool get_property_bool(jsobjtype obj, const char *name) ;
-jsobjtype get_property_object(jsobjtype parent, const char *name) ;
-jsobjtype get_property_function(jsobjtype parent, const char *name);
-jsobjtype get_array_element_object(jsobjtype obj, int idx) ;
-int set_property_string(jsobjtype obj, const char *name, const char *value) ;
-int set_property_number(jsobjtype obj, const char *name, int n) ;
-int set_property_float(jsobjtype obj, const char *name, double n) ;
-int set_property_bool(jsobjtype obj, const char *name, bool n) ;
-int set_property_object(jsobjtype parent, const char *name, jsobjtype child) ;
-jsobjtype instantiate_array(jsobjtype parent, const char *name) ;
-int set_array_element_object(jsobjtype array, int idx, jsobjtype child) ;
-jsobjtype instantiate_array_element(jsobjtype array, int idx, const char *classname) ;
-jsobjtype instantiate(jsobjtype parent, const char *name, const char *classname) ;
-int set_property_function(jsobjtype parent, const char *name, const char *body) ;
-int get_arraylength(jsobjtype a);
+void freeJavaContext(Frame *f) ;
+char *jsRunScriptResult(const Frame *f, jsobjtype obj, const char *str, const char *filename, int lineno) ;
+void jsRunScript(const Frame *f, jsobjtype obj, const char *str, const char *filename, int lineno) ;
+void jsRunData(const Frame *f, jsobjtype obj, const char *filename, int lineno);
+enum ej_proptype typeof_property(const Frame *f, jsobjtype obj, const char *name) ;
+#define handlerPresent(f, obj, name) (typeof_property(f, obj, name) == EJ_PROP_FUNCTION)
+bool has_property(const Frame *f, jsobjtype obj, const char *name) ;
+void delete_property(const Frame *f, jsobjtype obj, const char *name) ;
+char *get_property_string(const Frame *f, jsobjtype obj, const char *name) ;
+int get_property_number(const Frame *f, jsobjtype obj, const char *name) ;
+double get_property_float(const Frame *f, jsobjtype obj, const char *name) ;
+bool get_property_bool(const Frame *f, jsobjtype obj, const char *name) ;
+jsobjtype get_property_object(const Frame *f, jsobjtype parent, const char *name) ;
+jsobjtype get_property_function(const Frame *f, jsobjtype parent, const char *name);
+jsobjtype get_array_element_object(const Frame *f, jsobjtype obj, int idx) ;
+int set_property_string(const Frame *f, jsobjtype obj, const char *name, const char *value) ;
+int set_property_number(const Frame *f, jsobjtype obj, const char *name, int n) ;
+int set_property_float(const Frame *f, jsobjtype obj, const char *name, double n) ;
+int set_property_bool(const Frame *f, jsobjtype obj, const char *name, bool n) ;
+int set_property_object(const Frame *f, jsobjtype parent, const char *name, jsobjtype child) ;
+jsobjtype instantiate_array(const Frame *f, jsobjtype parent, const char *name) ;
+int set_array_element_object(const Frame *f, jsobjtype array, int idx, jsobjtype child) ;
+jsobjtype instantiate_array_element(const Frame *f, jsobjtype array, int idx, const char *classname) ;
+jsobjtype instantiate(const Frame *f, jsobjtype parent, const char *name, const char *classname) ;
+int set_property_function(const Frame *f, jsobjtype parent, const char *name, const char *body) ;
+int get_arraylength(const Frame *f, jsobjtype a);
 void setupJavaDom(void) ;
-char *get_property_url(jsobjtype owner, bool action) ;
+char *get_property_url(const Frame *f, jsobjtype owner, bool action) ;
 void rebuildSelectors(void);
-bool run_function_bool(jsobjtype obj, const char *name);
-jsobjtype create_event(jsobjtype parent, const char *evname);
-void unlink_event(jsobjtype parent);
-bool run_event_bool(jsobjtype obj, const char *pname, const char *evname, jsobjtype evobj);
-int run_function_onearg(jsobjtype obj, const char *name, jsobjtype o);
-void run_function_onestring(jsobjtype parent, const char *name, const char *s);
+bool run_function_bool(const Frame *f, jsobjtype obj, const char *name);
+jsobjtype create_event(const Frame *f, jsobjtype parent, const char *evname);
+void unlink_event(const Frame *f, jsobjtype parent);
+bool run_event_bool(const Frame *f, jsobjtype obj, const char *pname, const char *evname);
+int run_function_onearg(const Frame *f, jsobjtype obj, const char *name, jsobjtype o);
+void run_function_onestring(const Frame *f, jsobjtype parent, const char *name, const char *s);
 void set_basehref(const char *b);
 
 /* sourcefile=fetchmail.c */
@@ -144,9 +154,9 @@ bool tagHandler(int seqno, const char *name);
 void jSideEffects(void);
 void jSyncup(bool fromtimer);
 void jClearSync(void);
-void htmlMetaHelper(struct htmlTag *t);
-void prepareScript(struct htmlTag *t);
-void runScriptsPending(void);
+void htmlMetaHelper(Tag *t);
+void prepareScript(Tag *t);
+void runScriptsPending(bool startbrowse);
 void preFormatCheck(int tagno, bool * pretag, bool * slash) ;
 char *htmlParse(char *buf, int remote) ;
 bool htmlTest(void) ;
@@ -154,17 +164,17 @@ void infShow(int tagno, const char *search) ;
 void itext(void);
 bool infReplace(int tagno, const char *newtext, bool notify);
 bool infPush(int tagno, char **post_string);
-struct htmlTag *tagFromJavaVar(jsobjtype v);
+Tag *tagFromJavaVar(jsobjtype v);
 void javaSubmitsForm(jsobjtype v, bool reset);
-bool bubble_event(const struct htmlTag *t, const char *name);
+bool bubble_event(const Tag *t, const char *name);
 void runningError(int msg, ...);
 void rerender(bool notify);
 void delTags(int startRange, int endRange);
 void runOnload(void);
 void javaSetsTimeout(int n, const char *jsrc, jsobjtype to, bool isInterval);
-void scriptSetsTimeout( struct htmlTag *t);
+void scriptSetsTimeout( Tag *t);
 bool timerWait(int *delay_sec, int *delay_ms);
-void delTimers(struct ebFrame *f);
+void delTimers(Frame *f);
 void runTimer(void);
 void javaOpensWindow(const char *href, const char *name) ;
 void javaSetsLinkage(bool after, char type, jsobjtype p, const char *rest);
@@ -174,24 +184,24 @@ void html2nodes(const char *htmltext, bool startpage);
 
 /* sourcefile=decorate.c */
 void traverseAll(int start);
-const char *attribVal(const struct htmlTag *t, const char *name);
-struct htmlTag *findOpenTag(struct htmlTag *t, int action);
-struct htmlTag *findOpenList(struct htmlTag *t);
-void formControl(struct htmlTag *t, bool namecheck);
-void htmlInputHelper(struct htmlTag *t);
-char *displayOptions(const struct htmlTag *sel) ;
+const char *attribVal(const Tag *t, const char *name);
+Tag *findOpenTag(Tag *t, int action);
+Tag *findOpenList(Tag *t);
+void formControl(Tag *t, bool namecheck);
+void htmlInputHelper(Tag *t);
+char *displayOptions(const Tag *sel) ;
 void prerender(int start);
-jsobjtype instantiate_url(jsobjtype parent, const char *name, const char *url) ;
+jsobjtype instantiate_url(const Frame *f, jsobjtype parent, const char *name, const char *url) ;
 char *render(int start);
 void decorate(int start);
 void freeTags(struct ebWindow *w) ;
-struct htmlTag *newTag(const char *tagname) ;
+Tag *newTag(const Frame *f, const char *tagname) ;
 void initTagArray(void);
 void tag_gc(void);
 void tag_gc(void);
-void htmlNodesIntoTree(int start, struct htmlTag *attach);
-void underKill(struct htmlTag *t);
-void killTag(struct htmlTag *t);
+void htmlNodesIntoTree(int start, Tag *attach);
+void underKill(Tag *t);
+void killTag(Tag *t);
 void html_from_setter( jsobjtype innerParent, const char *h);
 
 /* sourcefile=http.c */
@@ -211,7 +221,7 @@ int bg_jobs(bool iponly);
 CURLcode setCurlURL(CURL * h, const char *url);
 bool frameExpand(bool expand, int ln1, int ln2);
 int frameExpandLine(int ln, jsobjtype fo);
-struct htmlTag *line2frame(int ln);
+Tag *line2frame(int ln);
 bool reexpandFrame(void);
 bool frameSecurityFile(const char *thisfile);
 void unframe(jsobjtype fobj, jsobjtype newdoc);
@@ -394,49 +404,52 @@ char *decodePostData(const char *data, const char *name, int seqno) ;
 void decodeMailURL(const char *url, char **addr_p, char **subj_p, char **body_p) ;
 bool patternMatchURL(const char *url, const char *pattern);
 
-/* sourcefile=jseng-duk.c */
-void connectTagObject(struct htmlTag *t, jsobjtype p);
-void disconnectTagObject(struct htmlTag *t);
-int js_main(void);
-// the native versions of the api functions in ebjs.c
-void createJavaContext_nat(void);
-void freeJavaContext_nat(jsobjtype cx);
-enum ej_proptype typeof_property_nat(jsobjtype obj, const char *name) ;
-bool has_property_nat(jsobjtype obj, const char *name) ;
-void delete_property_nat(jsobjtype obj, const char *name) ;
-char *get_property_string_nat(jsobjtype obj, const char *name) ;
-int get_property_number_nat(jsobjtype parent, const char *name) ;
-double get_property_float_nat(jsobjtype parent, const char *name) ;
-bool get_property_bool_nat(jsobjtype parent, const char *name) ;
-jsobjtype get_property_object_nat(jsobjtype parent, const char *name) ;
-jsobjtype get_property_function_nat(jsobjtype parent, const char *name) ;
-jsobjtype get_array_element_object_nat(jsobjtype obj, int idx) ;
-int set_property_string_nat(jsobjtype obj, const char *name, const char *value) ;
-int set_property_number_nat(jsobjtype obj, const char *name, int n) ;
-int set_property_float_nat(jsobjtype obj, const char *name, double n) ;
-int set_property_bool_nat(jsobjtype obj, const char *name, bool n) ;
-int set_property_object_nat(jsobjtype parent, const char *name, jsobjtype child) ;
-jsobjtype instantiate_array_nat(jsobjtype parent, const char *name) ;
-int set_array_element_object_nat(jsobjtype array, int idx, jsobjtype child) ;
-jsobjtype instantiate_array_element_nat(jsobjtype array, int idx, const char *classname) ;
-jsobjtype instantiate_nat(jsobjtype parent, const char *name, const char *classname) ;
-int set_property_function_nat(jsobjtype parent, const char *name, const char *body) ;
-int get_arraylength_nat(jsobjtype a);
-bool run_function_bool_nat(jsobjtype obj, const char *name);
-int run_function_onearg_nat(jsobjtype obj, const char *name, jsobjtype o);
-void run_function_onestring_nat(jsobjtype parent, const char *name, const char *s);
-char *run_script_nat(const char *s);
-void run_data_nat(jsobjtype o);
-void put_data_nat(jsobjtype cx, jsobjtype o, const char *s);
-
 /* sourcefile=css.c */
 void writeShortCache(void);
 bool matchMedia(char *t);
 void cssDocLoad(jsobjtype thisobj, char *s, bool pageload);
-void cssFree(struct ebFrame *f);
+void cssFree(Frame *f);
 jsobjtype querySelectorAll(const char *selstring, jsobjtype topobj);
 jsobjtype querySelector(const char *selstring, jsobjtype topobj);
 bool querySelector0(const char *selstring, jsobjtype topobj);
 void cssApply(jsobjtype thisobj, jsobjtype node, jsobjtype destination);
 void cssText(jsobjtype node, const char *rulestring);
 
+#ifdef __cplusplus
+}
+#endif
+
+/* sourcefile=jseng-duk.c */
+void connectTagObject(Tag *t, jsobjtype p);
+void disconnectTagObject(Tag *t);
+int js_main(void);
+// the native versions of the api functions in ebjs.c
+void createJavaContext_0(Frame *f);
+void freeJavaContext_0(jsobjtype cx);
+enum ej_proptype typeof_property_0(jsobjtype cx, jsobjtype obj, const char *name) ;
+bool has_property_0(jsobjtype cx, jsobjtype obj, const char *name) ;
+void delete_property_0(jsobjtype cx, jsobjtype obj, const char *name) ;
+char *get_property_string_0(jsobjtype cx, jsobjtype obj, const char *name) ;
+int get_property_number_0(jsobjtype cx, jsobjtype parent, const char *name) ;
+double get_property_float_0(jsobjtype cx, jsobjtype parent, const char *name) ;
+bool get_property_bool_0(jsobjtype cx, jsobjtype parent, const char *name) ;
+jsobjtype get_property_object_0(jsobjtype cx, jsobjtype parent, const char *name) ;
+jsobjtype get_property_function_0(jsobjtype cx, jsobjtype parent, const char *name) ;
+jsobjtype get_array_element_object_0(jsobjtype cx, jsobjtype obj, int idx) ;
+int set_property_string_0(jsobjtype cx, jsobjtype obj, const char *name, const char *value) ;
+int set_property_number_0(jsobjtype cx, jsobjtype obj, const char *name, int n) ;
+int set_property_float_0(jsobjtype cx, jsobjtype obj, const char *name, double n) ;
+int set_property_bool_0(jsobjtype cx, jsobjtype obj, const char *name, bool n) ;
+int set_property_object_0(jsobjtype cx, jsobjtype parent, const char *name, jsobjtype child) ;
+jsobjtype instantiate_array_0(jsobjtype cx, jsobjtype parent, const char *name) ;
+int set_array_element_object_0(jsobjtype cx, jsobjtype array, int idx, jsobjtype child) ;
+jsobjtype instantiate_array_element_0(jsobjtype cx, jsobjtype array, int idx, const char *classname) ;
+jsobjtype instantiate_0(jsobjtype cx, jsobjtype parent, const char *name, const char *classname) ;
+int set_property_function_0(jsobjtype cx, jsobjtype parent, const char *name, const char *body) ;
+int get_arraylength_0(jsobjtype cx, jsobjtype a);
+bool run_function_bool_0(jsobjtype cx, jsobjtype obj, const char *name);
+int run_function_onearg_0(jsobjtype cx, jsobjtype obj, const char *name, jsobjtype o);
+void run_function_onestring_0(jsobjtype cx, jsobjtype parent, const char *name, const char *s);
+char *run_script_0(jsobjtype cx, const char *s);
+void run_data_0(jsobjtype cx, jsobjtype o);
+
