@@ -1,2 +1,2 @@
 // Use this as a stub if you don't need deminimization, and are not including third.js
-mw0.compiled = true;
+mw$.compiled = true;
