@@ -141,15 +141,20 @@ static int mailstring_l;
 static char *mailbox_url, *message_url;
 
 static int imapfetch = 100;
+static bool earliest;
 
 static void setLimit(const char *t)
 {
+	char early = false;
 	skipWhite(&t);
+	if(*t == '-')
+		early = true, ++t;
 	if (!isdigit(*t)) {
 		i_puts(MSG_NumberExpected);
 		return;
 	}
 	imapfetch = atoi(t);
+	earliest = early;
 	if (imapfetch < 10)
 		imapfetch = 10;
 	i_printf(MSG_FetchN, imapfetch);
@@ -657,7 +662,7 @@ imap_done:
 			if (!imapSearch(handle, f, inputline))
 				goto action;
 			if (f->nmsgs > f->nfetch)
-				i_printf(MSG_ShowLast, f->nfetch, f->nmsgs);
+				i_printf(MSG_ShowLast + earliest, f->nfetch, f->nmsgs);
 			else
 				i_printf(MSG_MessagesX, f->nmsgs);
 			goto showmessages;
@@ -996,6 +1001,12 @@ doref:
 doflags:
 /* flags, mostly looking for has this been read */
 		u = strstr(t, "FLAGS (");
+		if(!u) {
+// sometimes flags and stuff comes at the beginning.
+// It should be somewhere!
+			t = mailstring;
+			u = strstr(t, "FLAGS (");
+		}
 		if (!u)
 			goto dodate;
 		t = u + 7;
@@ -1067,7 +1078,7 @@ static void examineFolder(CURL * handle, struct FOLDER *f, bool dostats)
 	if (f->nfetch > imapfetch)
 		f->nfetch = imapfetch;
 	f->start = 1;
-	if (f->nmsgs > f->nfetch)
+	if (f->nmsgs > f->nfetch && !earliest)
 		f->start += (f->nmsgs - f->nfetch);
 
 	t = strstr(mailstring, "UIDNEXT ");
@@ -1105,7 +1116,7 @@ static void examineFolder(CURL * handle, struct FOLDER *f, bool dostats)
 	envelopes(handle, f);
 
 	if (f->nmsgs > f->nfetch)
-		printf("showing last %d of %d messages\n", f->nfetch, f->nmsgs);
+		i_printf(MSG_ShowLast + earliest, f->nfetch, f->nmsgs);
 	else
 		i_printf(MSG_MessagesX, f->nmsgs);
 }				/* examineFolder */
@@ -2164,6 +2175,22 @@ static struct MHINFO *headerGlean(char *start, char *end)
 				stringAndBytes(&w->tolist, &w->tolen, s, t - s);
 			if (linetype == 'y')
 				stringAndBytes(&w->cclist, &w->cclen, s, t - s);
+			if (linetype == 's') {
+				int l1, l2;
+				++s;
+				l1 = strlen(w->subject);
+				l2 = t - s;
+				if(l1 + l2 > MHLINE)
+					l2 = MHLINE - l1;
+				strncpy(w->subject + l1, s, l2);
+				if (t == end - 1 || (t[1] != ' ' && t[1] != '\t')) {
+					vl = w->subject;
+					mhReformat(vl);
+					vr = vl + strlen(vl);
+					isoDecode(vl, &vr);
+					*vr = 0;
+				}
+			}
 			continue;
 		}
 
@@ -2193,8 +2220,10 @@ static struct MHINFO *headerGlean(char *start, char *end)
 /* This is sort of a switch statement on the word */
 		if (memEqualCI(s, "subject:", q - s)) {
 			linetype = 's';
-			if (w->subject[0])
+			if (w->subject[0]) {
+				linetype = 0;
 				continue;
+			}
 /* get rid of forward/reply prefixes */
 			for (q = vl; q < vr; ++q) {
 				static const char *const prefix[] = {
@@ -2220,13 +2249,14 @@ static struct MHINFO *headerGlean(char *start, char *end)
 				vr -= j;
 				--q;	/* try again */
 			}
-			isoDecode(vl, &vr);
 			strncpy(w->subject, vl, vr - vl);
-/* If the subject is really long, spreads onto the next line,
- * I'll just use ... */
-			if (t < end - 1 && (t[1] == ' ' || t[1] == '\t'))
-				strcat(w->subject, "...");
-			mhReformat(w->subject);
+			if (t == end - 1 || (t[1] != ' ' && t[1] != '\t')) {
+				vl = w->subject;
+				mhReformat(vl);
+				vr = vl + strlen(vl);
+				isoDecode(vl, &vr);
+				*vr = 0;
+			}
 			continue;
 		}
 
