@@ -14,7 +14,7 @@ blank spaces for messages no longer used.
 Each file lang/msg-* must have exactly this many lines.
 *********************************************************************/
 
-#define EdbrowseMessageCount 660
+#define EdbrowseMessageCount 662
 
 // English
 extern const char *msg_en[], ebrc_en[], qrg_en[];
@@ -411,7 +411,7 @@ enum {
 	MSG_FetchN,
 	MSG_NoAttachments,
 	MSG_EBRC_KeyInFunc,
-	MSG_ShowLast,
+	MSG_notused314,
 	MSG_FTPSession,
 	MSG_Push,
 	MSG_DBNotCompiled,
@@ -563,8 +563,8 @@ enum {
 	MSG_GlobalCommand2,
 	MSG_Doubleclick,
 	MSG_OptionSync,
-	MSG_notused466,
-	MSG_notused467,
+	MSG_MoveFileSystem,
+	MSG_MoveError,
 	MSG_GetLocalJS,
 	MSG_GetJS,
 	MSG_GetJS2,
@@ -652,7 +652,7 @@ enum {
 	MSG_ManyOpen,
 	MSG_InvalidSession,
 	MSG_InvalidLineNb,
-	MSG_notused555,
+	MSG_MetaChar,
 	MSG_SessionOutRange,
 	MSG_DoubleInit,
 	MSG_QuitNoActive,
@@ -682,7 +682,7 @@ enum {
 	MSG_JavaContextError,
 	MSG_JSDownForeground,
 	MSG_JSDownBackground,
-	MSG_notused585,
+	MSG_CopyFail,
 	MSG_TimersOff,
 	MSG_TimersOn,
 	MSG_MemAllocError,
@@ -758,4 +758,6 @@ enum {
 	MSG_SystemCmdFail,
 	MSG_ColorOff,
 	MSG_ColorOn,
+	MSG_ShowLast,
+	MSG_ShowFirst,
 };
