@@ -569,9 +569,28 @@ static char *mailTimeString(void)
 	static char buf[48];
 	struct tm *cur_tm;
 	time_t now;
+	static const char months[] =
+	    "Jan\0Feb\0Mar\0Apr\0May\0Jun\0Jul\0Aug\0Sep\0Oct\0Nov\0Dec";
+	static const char wdays[] = "Sun\0Mon\0Tue\0Wed\0Thu\0Fri\0Sat";
+
 	time(&now);
 	cur_tm = localtime(&now);
+
+/*********************************************************************
 	strftime(buf, sizeof(buf), "%a, %d %b %Y %H:%M:%S %z", cur_tm);
+That's what I use to do, but a user in France always got a date
+with French month and weekday, even though I setlocale(LC_TIME, "C");
+I changed C to en_US, still the French shines through.
+I don't understand it, so I just work around it.
+This is an internet standard, it's suppose to be English.
+*********************************************************************/
+
+	sprintf(buf, "%s, %02d %s ",
+		wdays + cur_tm->tm_wday * 4,
+		cur_tm->tm_mday, months + cur_tm->tm_mon * 4);
+// and strftime can do the rest.
+	strftime(buf + 12, sizeof(buf) - 12, "%Y %H:%M:%S %z", cur_tm);
+
 	return buf;
 }				/* mailTimeString */
 
@@ -690,6 +709,7 @@ static CURL *newSendmailHandle(const struct MACCOUNT *account,
 			       const char *outurl, const char *reply,
 			       struct curl_slist *recipients)
 {
+	static struct i_get g;
 	CURLcode res = CURLE_OK;
 	CURL *handle = curl_easy_init();
 	if (!handle) {
@@ -706,10 +726,7 @@ static CURL *newSendmailHandle(const struct MACCOUNT *account,
 	if (debugLevel >= 4)
 		curl_easy_setopt(handle, CURLOPT_VERBOSE, 1);
 	curl_easy_setopt(handle, CURLOPT_DEBUGFUNCTION, ebcurl_debug_handler);
-	res = curl_easy_setopt(handle, CURLOPT_CAINFO, sslCerts);
-	if (res != CURLE_OK) {
-		goto new_handle_cleanup;
-	}
+	curl_easy_setopt(handle, CURLOPT_DEBUGDATA, &g);
 
 	if (account->outssl == 2)
 		curl_easy_setopt(handle, CURLOPT_USE_SSL, CURLUSESSL_ALL);
@@ -741,7 +758,7 @@ static CURL *newSendmailHandle(const struct MACCOUNT *account,
 
 new_handle_cleanup:
 	if (res != CURLE_OK) {
-		ebcurl_setError(res, outurl);
+		ebcurl_setError(res, outurl, 0, emptyString);
 		curl_easy_cleanup(handle);
 		handle = NULL;
 	}
@@ -783,7 +800,7 @@ sendMailSMTP(const struct MACCOUNT *account, const char *reply,
 
 smtp_cleanup:
 	if (res != CURLE_OK)
-		ebcurl_setError(res, smtp_url);
+		ebcurl_setError(res, smtp_url, 0, emptyString);
 	if (handle)
 		curl_easy_cleanup(handle);
 	curl_slist_free_all(recipient_slist);
